@@ -3,7 +3,7 @@ import logging
 
 from ..deviceconfig import DeviceConfig
 from ..iotprotocol import IotProtocol
-from ..xortransport import XorTransport
+from ..protocol import BaseTransport
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -121,18 +121,83 @@ TIME_MODULE = {
     "set_timezone": None,
 }
 
+CLOUD_MODULE = {
+    "get_info": {
+        "username": "",
+        "server": "devs.tplinkcloud.com",
+        "binded": 0,
+        "cld_connection": 0,
+        "illegalType": -1,
+        "stopConnect": -1,
+        "tcspStatus": -1,
+        "fwDlPage": "",
+        "tcspInfo": "",
+        "fwNotifyType": 0,
+    }
+}
+
+
+AMBIENT_MODULE = {
+    "get_current_brt": {"value": 26, "err_code": 0},
+    "get_config": {
+        "devs": [
+            {
+                "hw_id": 0,
+                "enable": 0,
+                "dark_index": 1,
+                "min_adc": 0,
+                "max_adc": 2450,
+                "level_array": [
+                    {"name": "cloudy", "adc": 490, "value": 20},
+                    {"name": "overcast", "adc": 294, "value": 12},
+                    {"name": "dawn", "adc": 222, "value": 9},
+                    {"name": "twilight", "adc": 222, "value": 9},
+                    {"name": "total darkness", "adc": 111, "value": 4},
+                    {"name": "custom", "adc": 2400, "value": 97},
+                ],
+            }
+        ],
+        "ver": "1.0",
+        "err_code": 0,
+    },
+}
+
+
+MOTION_MODULE = {
+    "get_config": {
+        "enable": 0,
+        "version": "1.0",
+        "trigger_index": 2,
+        "cold_time": 60000,
+        "min_adc": 0,
+        "max_adc": 4095,
+        "array": [80, 50, 20, 0],
+        "err_code": 0,
+    }
+}
+
 
 class FakeIotProtocol(IotProtocol):
-    def __init__(self, info):
+    def __init__(self, info, fixture_name=None):
         super().__init__(
-            transport=XorTransport(
-                config=DeviceConfig("127.0.0.123"),
-            )
+            transport=FakeIotTransport(info, fixture_name),
         )
+
+    async def query(self, request, retry_count: int = 3):
+        """Implement query here so tests can still patch IotProtocol.query."""
+        resp_dict = await self._query(request, retry_count)
+        return resp_dict
+
+
+class FakeIotTransport(BaseTransport):
+    def __init__(self, info, fixture_name=None):
+        super().__init__(config=DeviceConfig("127.0.0.123"))
+        info = copy.deepcopy(info)
         self.discovery_data = info
+        self.fixture_name = fixture_name
         self.writer = None
         self.reader = None
-        proto = copy.deepcopy(FakeIotProtocol.baseproto)
+        proto = copy.deepcopy(FakeIotTransport.baseproto)
 
         for target in info:
             # print("target %s" % target)
@@ -164,6 +229,14 @@ class FakeIotProtocol(IotProtocol):
 
         self.proto = proto
 
+    @property
+    def default_port(self) -> int:
+        return 9999
+
+    @property
+    def credentials_hash(self) -> None:
+        return None
+
     def set_alias(self, x, child_ids=None):
         if child_ids is None:
             child_ids = []
@@ -219,6 +292,26 @@ class FakeIotProtocol(IotProtocol):
         self.proto["system"]["get_sysinfo"]["lighting_effect_state"] = dict(effect)
 
     def transition_light_state(self, state_changes, *args):
+        # Setting the light state on a device will turn off any active lighting effects.
+        # Unless it's just the brightness in which case it will update the brightness for
+        # the lighting effect
+        if lighting_effect_state := self.proto["system"]["get_sysinfo"].get(
+            "lighting_effect_state"
+        ):
+            if (
+                "hue" in state_changes
+                or "saturation" in state_changes
+                or "color_temp" in state_changes
+            ):
+                lighting_effect_state["enable"] = 0
+            elif (
+                lighting_effect_state["enable"] == 1
+                and state_changes.get("on_off") != 0
+                and (brightness := state_changes.get("brightness"))
+            ):
+                lighting_effect_state["brightness"] = brightness
+                return
+
         _LOGGER.debug("Setting light state to %s", state_changes)
         light_state = self.proto["system"]["get_sysinfo"]["light_state"]
 
@@ -246,9 +339,9 @@ class FakeIotProtocol(IotProtocol):
 
     def set_preferred_state(self, new_state, *args):
         """Implement set_preferred_state."""
-        self.proto["system"]["get_sysinfo"]["preferred_state"][
-            new_state["index"]
-        ] = new_state
+        self.proto["system"]["get_sysinfo"]["preferred_state"][new_state["index"]] = (
+            new_state
+        )
 
     def light_state(self, x, *args):
         light_state = self.proto["system"]["get_sysinfo"]["light_state"]
@@ -305,11 +398,13 @@ class FakeIotProtocol(IotProtocol):
             "set_brightness": set_hs220_brightness,
             "set_dimmer_transition": set_hs220_dimmer_transition,
         },
-        "smartlife.iot.LAS": {},
-        "smartlife.iot.PIR": {},
+        "smartlife.iot.LAS": AMBIENT_MODULE,
+        "smartlife.iot.PIR": MOTION_MODULE,
+        "cnCloud": CLOUD_MODULE,
+        "smartlife.iot.common.cloud": CLOUD_MODULE,
     }
 
-    async def query(self, request, port=9999):
+    async def send(self, request, port=9999):
         proto = self.proto
 
         # collect child ids from context
@@ -355,4 +450,10 @@ class FakeIotProtocol(IotProtocol):
         for target in request:
             response.update(get_response_for_module(target))
 
-        return response
+        return copy.deepcopy(response)
+
+    async def close(self) -> None:
+        pass
+
+    async def reset(self) -> None:
+        pass
