@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pytest
 from voluptuous import (
     All,
@@ -7,27 +9,29 @@ from voluptuous import (
     Schema,
 )
 
-from kasa import DeviceType, SmartBulb, SmartBulbPreset, SmartDeviceException
+from kasa import Device, DeviceType, IotLightPreset, KasaException, LightState, Module
+from kasa.iot import IotBulb, IotDimmer
 
 from .conftest import (
     bulb,
     bulb_iot,
     color_bulb,
     color_bulb_iot,
-    dimmable,
+    dimmable_iot,
     handle_turn_on,
     non_color_bulb,
-    non_dimmable,
+    non_dimmable_iot,
     non_variable_temp,
     turn_on,
     variable_temp,
     variable_temp_iot,
+    variable_temp_smart,
 )
-from .test_smartdevice import SYSINFO_SCHEMA
+from .test_iotdevice import SYSINFO_SCHEMA
 
 
 @bulb
-async def test_bulb_sysinfo(dev: SmartBulb):
+async def test_bulb_sysinfo(dev: Device):
     assert dev.sys_info is not None
     SYSINFO_SCHEMA_BULB(dev.sys_info)
 
@@ -40,17 +44,14 @@ async def test_bulb_sysinfo(dev: SmartBulb):
 
 
 @bulb
-async def test_state_attributes(dev: SmartBulb):
-    assert "Brightness" in dev.state_information
-    assert dev.state_information["Brightness"] == dev.brightness
-
-    assert "Is dimmable" in dev.state_information
-    assert dev.state_information["Is dimmable"] == dev.is_dimmable
+async def test_state_attributes(dev: Device):
+    assert "Cloud connection" in dev.state_information
+    assert isinstance(dev.state_information["Cloud connection"], bool)
 
 
 @bulb_iot
-async def test_light_state_without_update(dev: SmartBulb, monkeypatch):
-    with pytest.raises(SmartDeviceException):
+async def test_light_state_without_update(dev: IotBulb, monkeypatch):
+    with pytest.raises(KasaException):
         monkeypatch.setitem(
             dev._last_update["system"]["get_sysinfo"], "light_state", None
         )
@@ -58,33 +59,35 @@ async def test_light_state_without_update(dev: SmartBulb, monkeypatch):
 
 
 @bulb_iot
-async def test_get_light_state(dev: SmartBulb):
+async def test_get_light_state(dev: IotBulb):
     LIGHT_STATE_SCHEMA(await dev.get_light_state())
 
 
 @color_bulb
 @turn_on
-async def test_hsv(dev: SmartBulb, turn_on):
+async def test_hsv(dev: Device, turn_on):
+    light = dev.modules.get(Module.Light)
+    assert light
     await handle_turn_on(dev, turn_on)
-    assert dev.is_color
+    assert light.is_color
 
-    hue, saturation, brightness = dev.hsv
+    hue, saturation, brightness = light.hsv
     assert 0 <= hue <= 360
     assert 0 <= saturation <= 100
     assert 0 <= brightness <= 100
 
-    await dev.set_hsv(hue=1, saturation=1, value=1)
+    await light.set_hsv(hue=1, saturation=1, value=1)
 
     await dev.update()
-    hue, saturation, brightness = dev.hsv
+    hue, saturation, brightness = light.hsv
     assert hue == 1
     assert saturation == 1
     assert brightness == 1
 
 
 @color_bulb_iot
-async def test_set_hsv_transition(dev: SmartBulb, mocker):
-    set_light_state = mocker.patch("kasa.SmartBulb.set_light_state")
+async def test_set_hsv_transition(dev: IotBulb, mocker):
+    set_light_state = mocker.patch("kasa.iot.IotBulb._set_light_state")
     await dev.set_hsv(10, 10, 100, transition=1000)
 
     set_light_state.assert_called_with(
@@ -93,102 +96,137 @@ async def test_set_hsv_transition(dev: SmartBulb, mocker):
     )
 
 
+@bulb_iot
+async def test_light_set_state(dev: IotBulb, mocker):
+    """Testing setting LightState on the light module."""
+    light = dev.modules.get(Module.Light)
+    assert light
+    set_light_state = mocker.spy(dev, "_set_light_state")
+    state = LightState(light_on=True)
+    await light.set_state(state)
+
+    set_light_state.assert_called_with({"on_off": 1}, transition=None)
+    state = LightState(light_on=False)
+    await light.set_state(state)
+
+    set_light_state.assert_called_with({"on_off": 0}, transition=None)
+
+
 @color_bulb
 @turn_on
-async def test_invalid_hsv(dev: SmartBulb, turn_on):
+async def test_invalid_hsv(dev: Device, turn_on):
+    light = dev.modules.get(Module.Light)
+    assert light
     await handle_turn_on(dev, turn_on)
-    assert dev.is_color
+    assert light.is_color
 
     for invalid_hue in [-1, 361, 0.5]:
         with pytest.raises(ValueError):
-            await dev.set_hsv(invalid_hue, 0, 0)
+            await light.set_hsv(invalid_hue, 0, 0)  # type: ignore[arg-type]
 
     for invalid_saturation in [-1, 101, 0.5]:
         with pytest.raises(ValueError):
-            await dev.set_hsv(0, invalid_saturation, 0)
+            await light.set_hsv(0, invalid_saturation, 0)  # type: ignore[arg-type]
 
     for invalid_brightness in [-1, 101, 0.5]:
         with pytest.raises(ValueError):
-            await dev.set_hsv(0, 0, invalid_brightness)
+            await light.set_hsv(0, 0, invalid_brightness)  # type: ignore[arg-type]
 
 
 @color_bulb
-async def test_color_state_information(dev: SmartBulb):
+@pytest.mark.skip("requires color feature")
+async def test_color_state_information(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
     assert "HSV" in dev.state_information
-    assert dev.state_information["HSV"] == dev.hsv
+    assert dev.state_information["HSV"] == light.hsv
 
 
 @non_color_bulb
-async def test_hsv_on_non_color(dev: SmartBulb):
-    assert not dev.is_color
+async def test_hsv_on_non_color(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
+    assert not light.is_color
 
-    with pytest.raises(SmartDeviceException):
-        await dev.set_hsv(0, 0, 0)
-    with pytest.raises(SmartDeviceException):
-        print(dev.hsv)
+    with pytest.raises(KasaException):
+        await light.set_hsv(0, 0, 0)
+    with pytest.raises(KasaException):
+        print(light.hsv)
 
 
 @variable_temp
-async def test_variable_temp_state_information(dev: SmartBulb):
+@pytest.mark.skip("requires colortemp module")
+async def test_variable_temp_state_information(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
     assert "Color temperature" in dev.state_information
-    assert dev.state_information["Color temperature"] == dev.color_temp
-
-    assert "Valid temperature range" in dev.state_information
-    assert (
-        dev.state_information["Valid temperature range"] == dev.valid_temperature_range
-    )
+    assert dev.state_information["Color temperature"] == light.color_temp
 
 
 @variable_temp
 @turn_on
-async def test_try_set_colortemp(dev: SmartBulb, turn_on):
+async def test_try_set_colortemp(dev: Device, turn_on):
+    light = dev.modules.get(Module.Light)
+    assert light
     await handle_turn_on(dev, turn_on)
-    await dev.set_color_temp(2700)
+    await light.set_color_temp(2700)
     await dev.update()
-    assert dev.color_temp == 2700
+    assert light.color_temp == 2700
 
 
 @variable_temp_iot
-async def test_set_color_temp_transition(dev: SmartBulb, mocker):
-    set_light_state = mocker.patch("kasa.SmartBulb.set_light_state")
+async def test_set_color_temp_transition(dev: IotBulb, mocker):
+    set_light_state = mocker.patch("kasa.iot.IotBulb._set_light_state")
     await dev.set_color_temp(2700, transition=100)
 
     set_light_state.assert_called_with({"color_temp": 2700}, transition=100)
 
 
 @variable_temp_iot
-async def test_unknown_temp_range(dev: SmartBulb, monkeypatch, caplog):
+async def test_unknown_temp_range(dev: IotBulb, monkeypatch, caplog):
     monkeypatch.setitem(dev._sys_info, "model", "unknown bulb")
 
     assert dev.valid_temperature_range == (2700, 5000)
     assert "Unknown color temperature range, fallback to 2700-5000" in caplog.text
 
 
+@variable_temp_smart
+async def test_smart_temp_range(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
+    assert light.valid_temperature_range
+
+
 @variable_temp
-async def test_out_of_range_temperature(dev: SmartBulb):
+async def test_out_of_range_temperature(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
     with pytest.raises(ValueError):
-        await dev.set_color_temp(1000)
+        await light.set_color_temp(1000)
     with pytest.raises(ValueError):
-        await dev.set_color_temp(10000)
+        await light.set_color_temp(10000)
 
 
 @non_variable_temp
-async def test_non_variable_temp(dev: SmartBulb):
-    with pytest.raises(SmartDeviceException):
-        await dev.set_color_temp(2700)
+async def test_non_variable_temp(dev: Device):
+    light = dev.modules.get(Module.Light)
+    assert light
+    with pytest.raises(KasaException):
+        await light.set_color_temp(2700)
 
-    with pytest.raises(SmartDeviceException):
-        print(dev.valid_temperature_range)
+    with pytest.raises(KasaException):
+        print(light.valid_temperature_range)
 
-    with pytest.raises(SmartDeviceException):
-        print(dev.color_temp)
+    with pytest.raises(KasaException):
+        print(light.color_temp)
 
 
-@dimmable
+@dimmable_iot
 @turn_on
-async def test_dimmable_brightness(dev: SmartBulb, turn_on):
+async def test_dimmable_brightness(dev: IotBulb, turn_on):
+    assert isinstance(dev, (IotBulb, IotDimmer))
     await handle_turn_on(dev, turn_on)
-    assert dev.is_dimmable
+    assert dev._is_dimmable
 
     await dev.set_brightness(50)
     await dev.update()
@@ -199,12 +237,12 @@ async def test_dimmable_brightness(dev: SmartBulb, turn_on):
     assert dev.brightness == 10
 
     with pytest.raises(ValueError):
-        await dev.set_brightness("foo")
+        await dev.set_brightness("foo")  # type: ignore[arg-type]
 
 
 @bulb_iot
-async def test_turn_on_transition(dev: SmartBulb, mocker):
-    set_light_state = mocker.patch("kasa.SmartBulb.set_light_state")
+async def test_turn_on_transition(dev: IotBulb, mocker):
+    set_light_state = mocker.patch("kasa.iot.IotBulb._set_light_state")
     await dev.turn_on(transition=1000)
 
     set_light_state.assert_called_with({"on_off": 1}, transition=1000)
@@ -215,16 +253,16 @@ async def test_turn_on_transition(dev: SmartBulb, mocker):
 
 
 @bulb_iot
-async def test_dimmable_brightness_transition(dev: SmartBulb, mocker):
-    set_light_state = mocker.patch("kasa.SmartBulb.set_light_state")
+async def test_dimmable_brightness_transition(dev: IotBulb, mocker):
+    set_light_state = mocker.patch("kasa.iot.IotBulb._set_light_state")
     await dev.set_brightness(10, transition=1000)
 
-    set_light_state.assert_called_with({"brightness": 10}, transition=1000)
+    set_light_state.assert_called_with({"brightness": 10, "on_off": 1}, transition=1000)
 
 
-@dimmable
-async def test_invalid_brightness(dev: SmartBulb):
-    assert dev.is_dimmable
+@dimmable_iot
+async def test_invalid_brightness(dev: IotBulb):
+    assert dev._is_dimmable
 
     with pytest.raises(ValueError):
         await dev.set_brightness(110)
@@ -233,21 +271,21 @@ async def test_invalid_brightness(dev: SmartBulb):
         await dev.set_brightness(-100)
 
 
-@non_dimmable
-async def test_non_dimmable(dev: SmartBulb):
-    assert not dev.is_dimmable
+@non_dimmable_iot
+async def test_non_dimmable(dev: IotBulb):
+    assert not dev._is_dimmable
 
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         assert dev.brightness == 0
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await dev.set_brightness(100)
 
 
 @bulb_iot
 async def test_ignore_default_not_set_without_color_mode_change_turn_on(
-    dev: SmartBulb, mocker
+    dev: IotBulb, mocker
 ):
-    query_helper = mocker.patch("kasa.SmartBulb._query_helper")
+    query_helper = mocker.patch("kasa.iot.IotBulb._query_helper")
     # When turning back without settings, ignore default to restore the state
     await dev.turn_on()
     args, kwargs = query_helper.call_args_list[0]
@@ -259,32 +297,37 @@ async def test_ignore_default_not_set_without_color_mode_change_turn_on(
 
 
 @bulb_iot
-async def test_list_presets(dev: SmartBulb):
+async def test_list_presets(dev: IotBulb):
     presets = dev.presets
-    assert len(presets) == len(dev.sys_info["preferred_state"])
+    # Light strip devices may list some light effects along with normal presets but these
+    # are handled by the LightEffect module so exclude preferred states with id
+    raw_presets = [
+        pstate for pstate in dev.sys_info["preferred_state"] if "id" not in pstate
+    ]
+    assert len(presets) == len(raw_presets)
 
-    for preset, raw in zip(presets, dev.sys_info["preferred_state"]):
+    for preset, raw in zip(presets, raw_presets):
         assert preset.index == raw["index"]
-        assert preset.hue == raw["hue"]
         assert preset.brightness == raw["brightness"]
+        assert preset.hue == raw["hue"]
         assert preset.saturation == raw["saturation"]
         assert preset.color_temp == raw["color_temp"]
 
 
 @bulb_iot
-async def test_modify_preset(dev: SmartBulb, mocker):
+async def test_modify_preset(dev: IotBulb, mocker):
     """Verify that modifying preset calls the and exceptions are raised properly."""
     if not dev.presets:
         pytest.skip("Some strips do not support presets")
 
-    data = {
+    data: dict[str, int | None] = {
         "index": 0,
         "brightness": 10,
         "hue": 0,
         "saturation": 0,
         "color_temp": 0,
     }
-    preset = SmartBulbPreset(**data)
+    preset = IotLightPreset(**data)  # type: ignore[call-arg, arg-type]
 
     assert preset.index == 0
     assert preset.brightness == 10
@@ -293,11 +336,12 @@ async def test_modify_preset(dev: SmartBulb, mocker):
     assert preset.color_temp == 0
 
     await dev.save_preset(preset)
+    await dev.update()
     assert dev.presets[0].brightness == 10
 
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await dev.save_preset(
-            SmartBulbPreset(index=5, hue=0, brightness=0, saturation=0, color_temp=0)
+            IotLightPreset(index=5, hue=0, brightness=0, saturation=0, color_temp=0)  # type: ignore[call-arg]
         )
 
 
@@ -306,21 +350,21 @@ async def test_modify_preset(dev: SmartBulb, mocker):
     ("preset", "payload"),
     [
         (
-            SmartBulbPreset(index=0, hue=0, brightness=1, saturation=0),
+            IotLightPreset(index=0, hue=0, brightness=1, saturation=0),  # type: ignore[call-arg]
             {"index": 0, "hue": 0, "brightness": 1, "saturation": 0},
         ),
         (
-            SmartBulbPreset(index=0, brightness=1, id="testid", mode=2, custom=0),
+            IotLightPreset(index=0, brightness=1, id="testid", mode=2, custom=0),  # type: ignore[call-arg]
             {"index": 0, "brightness": 1, "id": "testid", "mode": 2, "custom": 0},
         ),
     ],
 )
-async def test_modify_preset_payloads(dev: SmartBulb, preset, payload, mocker):
+async def test_modify_preset_payloads(dev: IotBulb, preset, payload, mocker):
     """Test that modify preset payloads ignore none values."""
     if not dev.presets:
         pytest.skip("Some strips do not support presets")
 
-    query_helper = mocker.patch("kasa.SmartBulb._query_helper")
+    query_helper = mocker.patch("kasa.iot.IotBulb._query_helper")
     await dev.save_preset(preset)
     query_helper.assert_called_with(dev.LIGHT_SERVICE, "set_preferred_state", payload)
 
@@ -369,3 +413,10 @@ SYSINFO_SCHEMA_BULB = SYSINFO_SCHEMA.extend(
         ],
     }
 )
+
+
+@bulb
+def test_device_type_bulb(dev: Device):
+    if dev.is_light_strip:
+        pytest.skip("bulb has also lightstrips to test the api")
+    assert dev.device_type == DeviceType.Bulb
