@@ -1,5 +1,6 @@
 import json
 import logging
+import re
 import secrets
 import time
 from contextlib import nullcontext as does_not_raise
@@ -12,11 +13,11 @@ from ..aestransport import AesTransport
 from ..credentials import Credentials
 from ..deviceconfig import DeviceConfig
 from ..exceptions import (
-    AuthenticationException,
-    ConnectionException,
-    RetryableException,
-    SmartDeviceException,
-    TimeoutException,
+    AuthenticationError,
+    KasaException,
+    TimeoutError,
+    _ConnectionError,
+    _RetryableError,
 )
 from ..httpclient import HttpClient
 from ..iotprotocol import IotProtocol
@@ -68,7 +69,7 @@ async def test_protocol_retries_via_client_session(
     mocker.patch.object(protocol_class, "BACKOFF_SECONDS_AFTER_TIMEOUT", 0)
 
     config = DeviceConfig(host)
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await protocol_class(transport=transport_class(config=config)).query(
             DUMMY_QUERY, retry_count=retry_count
         )
@@ -80,11 +81,11 @@ async def test_protocol_retries_via_client_session(
 @pytest.mark.parametrize(
     "error, retry_expectation",
     [
-        (SmartDeviceException("dummy exception"), False),
-        (RetryableException("dummy exception"), True),
-        (TimeoutException("dummy exception"), True),
+        (KasaException("dummy exception"), False),
+        (_RetryableError("dummy exception"), True),
+        (TimeoutError("dummy exception"), True),
     ],
-    ids=("SmartDeviceException", "RetryableException", "TimeoutException"),
+    ids=("KasaException", "_RetryableError", "TimeoutError"),
 )
 @pytest.mark.parametrize("transport_class", [AesTransport, KlapTransport])
 @pytest.mark.parametrize("protocol_class", [IotProtocol, SmartProtocol])
@@ -97,7 +98,7 @@ async def test_protocol_retries_via_httpclient(
     mocker.patch.object(protocol_class, "BACKOFF_SECONDS_AFTER_TIMEOUT", 0)
 
     config = DeviceConfig(host)
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await protocol_class(transport=transport_class(config=config)).query(
             DUMMY_QUERY, retry_count=retry_count
         )
@@ -115,11 +116,11 @@ async def test_protocol_no_retry_on_connection_error(
     conn = mocker.patch.object(
         aiohttp.ClientSession,
         "post",
-        side_effect=AuthenticationException("foo"),
+        side_effect=AuthenticationError("foo"),
     )
     mocker.patch.object(protocol_class, "BACKOFF_SECONDS_AFTER_TIMEOUT", 0)
     config = DeviceConfig(host)
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await protocol_class(transport=transport_class(config=config)).query(
             DUMMY_QUERY, retry_count=5
         )
@@ -139,7 +140,7 @@ async def test_protocol_retry_recoverable_error(
         side_effect=aiohttp.ClientOSError("foo"),
     )
     config = DeviceConfig(host)
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         await protocol_class(transport=transport_class(config=config)).query(
             DUMMY_QUERY, retry_count=5
         )
@@ -159,7 +160,7 @@ async def test_protocol_reconnect(mocker, retry_count, protocol_class, transport
         nonlocal remaining
         remaining -= 1
         if remaining:
-            raise ConnectionException("Simulated connection failure")
+            raise _ConnectionError("Simulated connection failure")
 
         return mock_response
 
@@ -238,6 +239,71 @@ def test_encrypt_unicode():
     assert d == decrypted
 
 
+async def test_transport_decrypt(mocker):
+    """Test transport decryption."""
+    d = {"great": "success"}
+
+    seed = secrets.token_bytes(16)
+    auth_hash = KlapTransport.generate_auth_hash(Credentials("foo", "bar"))
+    encryption_session = KlapEncryptionSession(seed, seed, auth_hash)
+
+    transport = KlapTransport(config=DeviceConfig(host="127.0.0.1"))
+    transport._handshake_done = True
+    transport._session_expire_at = time.monotonic() + 60
+    transport._encryption_session = encryption_session
+
+    async def _return_response(url: URL, params=None, data=None, *_, **__):
+        encryption_session = KlapEncryptionSession(
+            transport._encryption_session.local_seed,
+            transport._encryption_session.remote_seed,
+            transport._encryption_session.user_hash,
+        )
+        seq = params.get("seq")
+        encryption_session._seq = seq - 1
+        encrypted, seq = encryption_session.encrypt(json.dumps(d))
+        seq = seq
+        return 200, encrypted
+
+    mocker.patch.object(HttpClient, "post", side_effect=_return_response)
+
+    resp = await transport.send(json.dumps({}))
+    assert d == resp
+
+
+async def test_transport_decrypt_error(mocker, caplog):
+    """Test that a decryption error raises a kasa exception."""
+    d = {"great": "success"}
+
+    seed = secrets.token_bytes(16)
+    auth_hash = KlapTransport.generate_auth_hash(Credentials("foo", "bar"))
+    encryption_session = KlapEncryptionSession(seed, seed, auth_hash)
+
+    transport = KlapTransport(config=DeviceConfig(host="127.0.0.1"))
+    transport._handshake_done = True
+    transport._session_expire_at = time.monotonic() + 60
+    transport._encryption_session = encryption_session
+
+    async def _return_response(url: URL, params=None, data=None, *_, **__):
+        encryption_session = KlapEncryptionSession(
+            secrets.token_bytes(16),
+            transport._encryption_session.remote_seed,
+            transport._encryption_session.user_hash,
+        )
+        seq = params.get("seq")
+        encryption_session._seq = seq - 1
+        encrypted, seq = encryption_session.encrypt(json.dumps(d))
+        seq = seq
+        return 200, encrypted
+
+    mocker.patch.object(HttpClient, "post", side_effect=_return_response)
+
+    with pytest.raises(
+        KasaException,
+        match=re.escape("Error trying to decrypt device 127.0.0.1 response:"),
+    ):
+        await transport.send(json.dumps({}))
+
+
 @pytest.mark.parametrize(
     "device_credentials, expectation",
     [
@@ -249,7 +315,7 @@ def test_encrypt_unicode():
         ),
         (
             Credentials("shouldfail", "shouldfail"),
-            pytest.raises(AuthenticationException),
+            pytest.raises(AuthenticationError),
         ),
     ],
     ids=("client", "blank", "kasa_setup", "shouldfail"),
@@ -323,14 +389,14 @@ async def test_handshake(
     async def _return_handshake_response(url: URL, params=None, data=None, *_, **__):
         nonlocal client_seed, server_seed, device_auth_hash
 
-        if str(url) == "http://127.0.0.1/app/handshake1":
+        if str(url) == "http://127.0.0.1:80/app/handshake1":
             client_seed = data
             seed_auth_hash = _sha256(
                 seed_auth_hash_calc1(client_seed, server_seed, device_auth_hash)
             )
 
             return _mock_response(200, server_seed + seed_auth_hash)
-        elif str(url) == "http://127.0.0.1/app/handshake2":
+        elif str(url) == "http://127.0.0.1:80/app/handshake2":
             seed_auth_hash = _sha256(
                 seed_auth_hash_calc2(client_seed, server_seed, device_auth_hash)
             )
@@ -350,7 +416,7 @@ async def test_handshake(
     assert protocol._transport._handshake_done is True
 
     response_status = 403
-    with pytest.raises(AuthenticationException):
+    with pytest.raises(KasaException):
         await protocol._transport.perform_handshake()
     assert protocol._transport._handshake_done is False
     await protocol.close()
@@ -367,14 +433,14 @@ async def test_query(mocker):
     async def _return_response(url: URL, params=None, data=None, *_, **__):
         nonlocal client_seed, server_seed, device_auth_hash, seq
 
-        if str(url) == "http://127.0.0.1/app/handshake1":
+        if str(url) == "http://127.0.0.1:80/app/handshake1":
             client_seed = data
             client_seed_auth_hash = _sha256(data + device_auth_hash)
 
             return _mock_response(200, server_seed + client_seed_auth_hash)
-        elif str(url) == "http://127.0.0.1/app/handshake2":
+        elif str(url) == "http://127.0.0.1:80/app/handshake2":
             return _mock_response(200, b"")
-        elif str(url) == "http://127.0.0.1/app/request":
+        elif str(url) == "http://127.0.0.1:80/app/request":
             encryption_session = KlapEncryptionSession(
                 protocol._transport._encryption_session.local_seed,
                 protocol._transport._encryption_session.remote_seed,
@@ -400,35 +466,81 @@ async def test_query(mocker):
 
 
 @pytest.mark.parametrize(
-    "response_status, expectation",
+    "response_status, credentials_match, expectation",
     [
-        ((403, 403, 403), pytest.raises(AuthenticationException)),
-        ((200, 403, 403), pytest.raises(AuthenticationException)),
-        ((200, 200, 403), pytest.raises(AuthenticationException)),
-        ((200, 200, 400), pytest.raises(SmartDeviceException)),
+        pytest.param(
+            (403, 403, 403),
+            True,
+            pytest.raises(KasaException),
+            id="handshake1-403-status",
+        ),
+        pytest.param(
+            (200, 403, 403),
+            True,
+            pytest.raises(KasaException),
+            id="handshake2-403-status",
+        ),
+        pytest.param(
+            (200, 200, 403),
+            True,
+            pytest.raises(_RetryableError),
+            id="request-403-status",
+        ),
+        pytest.param(
+            (200, 200, 400),
+            True,
+            pytest.raises(KasaException),
+            id="request-400-status",
+        ),
+        pytest.param(
+            (200, 200, 200),
+            False,
+            pytest.raises(AuthenticationError),
+            id="handshake1-wrong-auth",
+        ),
+        pytest.param(
+            (200, 200, 200),
+            secrets.token_bytes(16),
+            pytest.raises(KasaException),
+            id="handshake1-bad-auth-length",
+        ),
     ],
-    ids=("handshake1", "handshake2", "request", "non_auth_error"),
 )
-async def test_authentication_failures(mocker, response_status, expectation):
+async def test_authentication_failures(
+    mocker, response_status, credentials_match, expectation
+):
     client_seed = None
 
     server_seed = secrets.token_bytes(16)
     client_credentials = Credentials("foo", "bar")
-    device_auth_hash = KlapTransport.generate_auth_hash(client_credentials)
+    device_credentials = (
+        client_credentials if credentials_match else Credentials("bar", "foo")
+    )
+    device_auth_hash = KlapTransport.generate_auth_hash(device_credentials)
 
     async def _return_response(url: URL, params=None, data=None, *_, **__):
-        nonlocal client_seed, server_seed, device_auth_hash, response_status
+        nonlocal \
+            client_seed, \
+            server_seed, \
+            device_auth_hash, \
+            response_status, \
+            credentials_match
 
-        if str(url) == "http://127.0.0.1/app/handshake1":
+        if str(url) == "http://127.0.0.1:80/app/handshake1":
             client_seed = data
             client_seed_auth_hash = _sha256(data + device_auth_hash)
-
+            if credentials_match is not False and credentials_match is not True:
+                client_seed_auth_hash += credentials_match
             return _mock_response(
                 response_status[0], server_seed + client_seed_auth_hash
             )
-        elif str(url) == "http://127.0.0.1/app/handshake2":
-            return _mock_response(response_status[1], b"")
-        elif str(url) == "http://127.0.0.1/app/request":
+        elif str(url) == "http://127.0.0.1:80/app/handshake2":
+            client_seed = data
+            client_seed_auth_hash = _sha256(data + device_auth_hash)
+            return _mock_response(
+                response_status[1], server_seed + client_seed_auth_hash
+            )
+        elif str(url) == "http://127.0.0.1:80/app/request":
             return _mock_response(response_status[2], b"")
 
     mocker.patch.object(aiohttp.ClientSession, "post", side_effect=_return_response)
@@ -438,3 +550,14 @@ async def test_authentication_failures(mocker, response_status, expectation):
 
     with expectation:
         await protocol.query({})
+
+
+async def test_port_override():
+    """Test that port override sets the app_url."""
+    host = "127.0.0.1"
+    config = DeviceConfig(
+        host, credentials=Credentials("foo", "bar"), port_override=12345
+    )
+    transport = KlapTransport(config=config)
+
+    assert str(transport._app_url) == "http://127.0.0.1:12345/app"
